@@ -126,7 +126,7 @@ void Box_Data_Details::init_db_details(const Glib::ustring& strTableName, const
 
   get_field_primary_key_for_table(strTableName, m_field_primary_key);
 
-  Box_DB_Table::init_db_details(strTableName);
+  Box_Data::init_db_details(strTableName);
 }
 
 void Box_Data_Details::refresh_db_details(const Gnome::Gda::Value& primary_key_value)
@@ -145,12 +145,12 @@ void Box_Data_Details::fill_from_database_layout()
 
   //Remove existing child widgets:
   m_FlowTable.remove_all();
-                          
+
   Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
   if(document)
   {
     m_FlowTable.set_table(m_strTableName); //This allows portals to get full Relationship information
-    
+
     //This map of layout groups will also contain the field information from the database:
     Document_Glom::type_mapLayoutGroupSequence layout_groups = get_data_layout_groups("details");
     for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
@@ -175,74 +175,66 @@ void Box_Data_Details::fill_from_database()
     Box_DB_Table::fill_from_database();
 
     m_Fields = get_fields_to_show();
-    type_vecFields fieldsToGet = m_Fields;
+    type_vecLayoutFields fieldsToGet = m_Fields;
 
     if(!fieldsToGet.empty())
     {
       fill_from_database_layout(); //TODO: Only do this when the layout has changed.
 
-      //Add extra possibly-non-visible columns that we need:
-      fieldsToGet.push_back(m_field_primary_key); //Actually listFieldsToGet.
+      //Do not try to show the data if the user may not view it:
+      Privileges table_privs = get_current_privs(m_strTableName);
 
-      //g_warning("primary_key name = %s", m_field_primary_key.get_name().c_str());
-      const int index_primary_key = fieldsToGet.size() - 1;
+      //Enable/Disable record creation and deletion:
+      m_Button_New.set_sensitive(table_privs.m_create);
+      m_Button_Del.set_sensitive(table_privs.m_delete);
 
-      if(!GlomConversions::value_is_empty(m_primary_key_value)) //If there is a record to show:
+      if(table_privs.m_view)
       {
-        if(sharedconnection)
+        //Add extra possibly-non-visible columns that we need:
+        LayoutItem_Field layout_item;
+        layout_item.set_name(m_field_primary_key.get_name());
+        layout_item.m_field = m_field_primary_key;
+        fieldsToGet.push_back(layout_item);
+
+        //g_warning("primary_key name = %s", m_field_primary_key.get_name().c_str());
+        const int index_primary_key = fieldsToGet.size() - 1;
+
+        const Glib::ustring query = build_sql_select(m_strTableName, fieldsToGet, m_field_primary_key, m_primary_key_value);
+        Glib::RefPtr<Gnome::Gda::DataModel> result = Query_execute(query);
+
+        if(result && result->get_n_rows())
         {
-          Glib::RefPtr<Gnome::Gda::Connection> connection = sharedconnection->get_gda_connection();
-
-          Glib::ustring sql_part_fields;
-          for(type_vecFields::const_iterator iter =  fieldsToGet.begin(); iter != fieldsToGet.end(); ++iter)
+          const Document_Glom* pDoc = dynamic_cast<const Document_Glom*>(get_document());
+          if(pDoc)
           {
-            if(iter != fieldsToGet.begin())
-              sql_part_fields += ",";
+            //Get glom-specific field info:
+            //Document_Glom::type_vecFields vecFields = pDoc->get_table_fields(m_strTableName);
 
-            sql_part_fields += iter->get_name();
-          }
+            const int row_number = 0; //The only row.
+            const int cols_count = result->get_n_columns();
 
-          std::stringstream query;
-          query << "SELECT " << sql_part_fields << " FROM " << m_strTableName << " WHERE " << get_table_name() + "." + m_field_primary_key.get_name() << " = " << m_field_primary_key.sql(m_primary_key_value);
-          Glib::RefPtr<Gnome::Gda::DataModel> result = connection->execute_single_command(query.str());
+            //Get special possibly-non-visible field values:
+            if(index_primary_key < cols_count)
+              m_primary_key_value = result->get_value_at(index_primary_key, row_number);
 
-          if(result && result->get_n_rows())
-          {
-            const Document_Glom* pDoc = dynamic_cast<const Document_Glom*>(get_document());
-            if(pDoc)
+            //Get field values to show:
+            for(int i = 0; i < cols_count; ++i)
             {
-              //Get glom-specific field info:
-              //Document_Glom::type_vecFields vecFields = pDoc->get_table_fields(m_strTableName);
+              const LayoutItem_Field& layout_item = fieldsToGet[i];
 
-              const int row_number = 0; //The only row.
-              const int cols_count = result->get_n_columns();
-
-              //Get special possibly-non-visible field values:
-              if(index_primary_key < cols_count)
-                m_primary_key_value = result->get_value_at(index_primary_key, row_number);
-
-              //Get field values to show:
-              for(int i = 0; i < cols_count; ++i)
-              {
-                const Field& field = fieldsToGet[i];
-
-                //Field value:
-                Gnome::Gda::Value value = result->get_value_at(i, row_number);
-                m_FlowTable.set_field_value(field, value);
-              }
+              //Field value:
+              Gnome::Gda::Value value = result->get_value_at(i, row_number);
+              m_FlowTable.set_field_value(layout_item, value);
             }
           }
         }
       }
-      else
-      {
-        //Show blank record:
-        //TODO
-      }
     } //if(!fieldsToGet.empty())
 
     //fill_related();
 
+    set_unstored_data(false);
+
     fill_end();
 
   }
@@ -347,16 +339,10 @@ void Box_Data_Details::on_button_new()
     if(fieldinfo.get_auto_increment()) //If the primary key is an auto-increment:
     {
       //Just make a new record, and show it:
-      guint generated_id = generate_next_auto_increment(m_strTableName, m_field_primary_key.get_name()); //TODO: This should return a Gda::Value
+      Gnome::Gda::Value primary_key_value = generate_next_auto_increment(m_strTableName, m_field_primary_key.get_name()); //TODO: This should return a Gda::Value
 
-      //TODO: Create the Gda::Value directly from the integer.
-      bool parsed = false; 
-      Gnome::Gda::Value primary_key_value = GlomConversions::parse_value(m_field_primary_key.get_glom_type(), util_string_from_decimal(generated_id), parsed);
-      if(parsed)
-      {
-        record_new(false /* use entered field data */, primary_key_value);
-        refresh_db_details(primary_key_value);
-      }
+      record_new(false /* use entered field data */, primary_key_value);
+      refresh_db_details(primary_key_value);
     }
     else
     {
@@ -373,7 +359,9 @@ void Box_Data_Details::on_button_del()
   if( GlomConversions::value_is_empty(get_primary_key_value()) )
   {
     //Tell user that a primary key is needed to delete a record:
-    Gtk::MessageDialog dialog(gettext("This record can not be deleted because there is no primary key."));
+    Gtk::MessageDialog dialog(gettext("<b>No primary key value.</b>"), true);
+    dialog.set_secondary_text(gettext("This record can not be deleted because there is no primary key."));
+    dialog.set_transient_for(*get_app_window());
     dialog.run();
   }
   else
@@ -413,12 +401,12 @@ void Box_Data_Details::on_button_nav_last()
     signal_nav_last().emit();
 }
 
-Gnome::Gda::Value Box_Data_Details::get_entered_field_data(const Field& field) const
+Gnome::Gda::Value Box_Data_Details::get_entered_field_data(const LayoutItem_Field& field) const
 {
   return m_FlowTable.get_field_value(field);
 }
 
-void Box_Data_Details::set_entered_field_data(const Field& field, const Gnome::Gda::Value& value)
+void Box_Data_Details::set_entered_field_data(const LayoutItem_Field& field, const Gnome::Gda::Value& value)
 {
   m_FlowTable.set_field_value(field, value);
 }
@@ -521,27 +509,74 @@ void Box_Data_Details::on_flowtable_layout_changed()
   fill_from_database();
 }
 
-void Box_Data_Details::on_flowtable_field_edited(const Glib::ustring& id, const Gnome::Gda::Value& field_value)
+void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_field, const Gnome::Gda::Value& field_value)
 {
   if(m_ignore_signals)
     return;
 
-  const Glib::ustring strFieldName = id;
+  const Glib::ustring strFieldName = layout_field.get_name();
 
   Gnome::Gda::Value primary_key_value = get_primary_key_value();
   if(!GlomConversions::value_is_empty(primary_key_value)) //If there is a primary key value:
   {
-    Field field;
-    bool test = get_field(strFieldName, field);
-    if(!test)
-      g_warning("Box_Data_Details::on_flowtable_field_edited: field not found");
+    Glib::ustring table_name;
+    Field primary_key_field;
+    Gnome::Gda::Value primary_key_value;
+
+    if(!layout_field.get_has_relationship_name())
+    {
+      table_name = get_table_name();
+      primary_key_field = m_field_primary_key;
+      primary_key_value = get_primary_key_value();
+    }
+    else
+    {
+      //If it's a related field then discover the actual table that it's in,
+      //plus how to identify the record in that table.
+      const Glib::ustring relationship_name = layout_field.get_relationship_name();
+
+      Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
+
+      Relationship relationship;
+      bool test = document->get_relationship(get_table_name(), relationship_name, relationship);
+      if(test)
+      {
+        table_name = relationship.get_to_table();
+        const Glib::ustring to_field_name = relationship.get_to_field();
+        //Get the key field in the other table (the table that we will change)
+        bool test = get_fields_for_table_one_field(table_name, to_field_name, primary_key_field); //TODO_Performance.
+        if(test)
+        {
+          //Get the value of the corresponding key in the current table (that identifies the record in the table that we will change)
+          LayoutItem_Field layout_item;
+          layout_item.set_name(relationship.get_from_field());
+
+          primary_key_value = get_entered_field_data(layout_item);
+
+          bool test = add_related_record_for_field(layout_field, relationship, primary_key_field, primary_key_value);
+          if(!test)
+            return;
+
+          //Get the new primary_key_value if it has been created:
+          primary_key_value = get_entered_field_data(layout_item);
+
+          //Now that the related record exists, the following code to set the value of the other field in the related field can succeed.
+        }
+        else
+        {
+          g_warning("Box_Data_Details::on_flowtable_field_edited(): key not found for edited related field.");
+        }
+      }
+    }
+
+    const Field& field = layout_field.m_field;
 
     //Update the field in the record (the record with this primary key):
     try
     {
-      Glib::ustring strQuery = "UPDATE " + m_strTableName;
-      strQuery += " SET " +  /* get_table_name() + "." +*/ strFieldName + " = " + field.sql(field_value);
-      strQuery += " WHERE " + get_table_name() + "." + m_field_primary_key.get_name() + " = " + m_field_primary_key.sql(get_primary_key_value());
+      Glib::ustring strQuery = "UPDATE " + table_name;
+      strQuery += " SET " +  /* table_name + "." + postgres does not seem to like the table name here */ strFieldName + " = " + field.sql(field_value);
+      strQuery += " WHERE " + table_name + "." + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
       bool bTest = Query_execute(strQuery);
 
       if(!bTest)
@@ -552,10 +587,15 @@ void Box_Data_Details::on_flowtable_field_edited(const Glib::ustring& id, const
       else
       {
         //Set the value in all instances of this field in the layout (The field might be on the layout more than once):
-        m_FlowTable.set_field_value(field, field_value);
-        
+        m_FlowTable.set_field_value(layout_field, field_value);
+
         //Get-and-set values for lookup fields, if this field triggers those relationships:
-        do_lookups(field, field_value, m_field_primary_key, primary_key_value);
+        do_lookups(layout_field, field_value, primary_key_field, primary_key_value);
+
+        //Show new values for related fields:
+        refresh_related_fields(layout_field, field_value, primary_key_field, primary_key_value);
+
+        //TODO: Display new values for related fields.
 
         //If this is a foreign key then refresh the related records:
         /*
@@ -592,7 +632,9 @@ void Box_Data_Details::on_flowtable_field_edited(const Glib::ustring& id, const
       if(strFieldName == m_field_primary_key.get_name()) //If edited field is the primary key.
       {
         //Warn user that they can't choose their own primary key:
-        Gtk::MessageDialog dialog ("The primary key is auto-incremented.\n You may not enter your own primary key value.");
+        Gtk::MessageDialog dialog(gettext("<b>Primary key auto increments</b>"), true);
+        dialog.set_secondary_text(gettext("The primary key is auto-incremented.\n You may not enter your own primary key value."));
+        dialog.set_transient_for(*get_app_window());
         dialog.run();
       }
     }
@@ -619,16 +661,63 @@ void Box_Data_Details::on_flowtable_field_edited(const Glib::ustring& id, const
   } //if(get_primary_key_value().size())
 }
 
-void Box_Data_Details::do_lookups(const Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
+void Box_Data_Details::refresh_related_fields(const LayoutItem_Field& field_changed, const Gnome::Gda::Value& /* field_value */, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
 {
+  if(field_changed.get_has_relationship_name())
+    return; //TODO: Handle these too.
+
+  //Get values for lookup fields, if this field triggers those relationships:
+  //TODO_performance: There is a LOT of iterating and copying here.
+  const Glib::ustring strFieldName = field_changed.get_name();
+  type_vecLayoutFields fieldsToGet = get_related_fields(strFieldName);
+
+  if(!fieldsToGet.empty())
+  {
+    const Glib::ustring query = build_sql_select(m_strTableName, fieldsToGet, primary_key, primary_key_value);
+
+    Glib::RefPtr<Gnome::Gda::DataModel> result = Query_execute(query);
+    if(!result)
+      handle_error();
+    else
+    {
+      //Field contents:
+      if(result->get_n_rows())
+      {
+        type_vecLayoutFields::const_iterator iterFields = fieldsToGet.begin();
+
+        guint cols_count = result->get_n_columns();
+        for(guint uiCol = 0; uiCol < cols_count; uiCol++)
+        {
+          const Gnome::Gda::Value value = result->get_value_at(uiCol, 0 /* row */);
+          const LayoutItem_Field& layout_item = *iterFields;
+
+          //g_warning("list fill: field_name=%s", iterFields->get_name().c_str());
+          //g_warning("  value_as_string=%s", value.to_string().c_str());
+
+          m_FlowTable.set_field_value(layout_item, value);
+
+          ++iterFields;
+        }
+      }
+    }
+  }
+}
+
+void Box_Data_Details::do_lookups(const LayoutItem_Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
+{
+   if(field_changed.get_has_relationship_name())
+    return; //TODO: Handle these too.
+
    //Get values for lookup fields, if this field triggers those relationships:
    //TODO_performance: There is a LOT of iterating and copying here.
    const Glib::ustring strFieldName = field_changed.get_name();
    type_list_lookups lookups = get_lookup_fields(strFieldName);
    for(type_list_lookups::const_iterator iter = lookups.begin(); iter != lookups.end(); ++iter)
    {
+     const LayoutItem_Field& layout_Item = iter->first;
+
      const Relationship relationship = iter->second;
-     const Field field_lookup = iter->first;
+     const Field& field_lookup = layout_Item.m_field;
      const Glib::ustring field_lookup_name = field_lookup.get_name();
 
      Field field_source;
@@ -638,7 +727,10 @@ void Box_Data_Details::do_lookups(const Field& field_changed, const Gnome::Gda::
        Gnome::Gda::Value value = get_lookup_value(iter->second /* relationship */,  field_source /* the field to look in to get the value */, field_value /* Value of to and from fields */);
 
        //Add it to the view:
-       m_FlowTable.set_field_value(field_lookup_name, value);
+       LayoutItem_Field layout_item;
+       layout_item.set_name(field_lookup_name);
+       //TODO? layout_item.set_relationship_name();
+       m_FlowTable.set_field_value(layout_item, value);
 
        //Add it to the database (even if it is not shown in the view)
        Glib::ustring strQuery = "UPDATE " + m_strTableName;
@@ -663,6 +755,3 @@ bool Box_Data_Details::get_field_primary_key(Field& field) const
   return false;
 }
 
-
-
-
