@@ -93,7 +93,7 @@ void Box_DB_Table_Definition::fill_field_row(const Gtk::TreeModel::iterator& ite
   //Type:
   Field::glom_field_type fieldType = Field::get_glom_type_for_gda_type(fieldinfo.get_gdatype()); //Could be TYPE_INVALID if the gda type is not one of ours.
 
-  Glib::ustring strType = Field::get_type_name( fieldType );
+  Glib::ustring strType = Field::get_type_name_ui( fieldType );
   m_AddDel.set_value(iter, m_colType, strType);
 
   //Unique:
@@ -115,12 +115,12 @@ void Box_DB_Table_Definition::fill_from_database()
     //Fields:
     m_AddDel.remove_all();
 
-    Field::type_map_type_names mapFieldTypes = Field::get_type_names();
+    Field::type_map_type_names mapFieldTypes = Field::get_type_names_ui();
 
-    for(type_vecFields::iterator iter = m_Fields.begin(); iter != m_Fields.end(); iter++)
-    {       
+    for(type_vecFields::iterator iter = m_vecFields.begin(); iter != m_vecFields.end(); iter++)
+    {
       const Field& field = *iter;
-             
+
       //Name:
       Gtk::TreeModel::iterator iter= m_AddDel.add_item(field.get_name());
       fill_field_row(iter, field);
@@ -170,7 +170,7 @@ void Box_DB_Table_Definition::on_adddel_delete(const Gtk::TreeModel::iterator& r
   Gtk::TreeModel::iterator iterAfterEnd = rowEnd;
   if(iterAfterEnd != m_AddDel.get_model()->children().end())
     ++iterAfterEnd;
-    
+
   for(Gtk::TreeModel::iterator iter = rowStart; iter != iterAfterEnd; ++iter)
   {
     Glib::ustring strName = m_AddDel.get_value_key(iter);
@@ -192,18 +192,18 @@ void Box_DB_Table_Definition::on_adddel_changed(const Gtk::TreeModel::iterator&
   {
     //Glom-specific stuff: //TODO_portiter
     const Glib::ustring strFieldNameBeingEdited = m_AddDel.get_value_key(row);
-    
+ 
     pDoc->get_field(m_strTableName, strFieldNameBeingEdited, m_Field_BeingEdited);
 
     //Get DB field info: (TODO: This might be unnecessary).
-    type_vecFields::const_iterator iterFind = std::find_if( m_Fields.begin(), m_Fields.end(), predicate_FieldHasName<Field>(strFieldNameBeingEdited) );
-    if(iterFind != m_Fields.end()) //If it was found:
+    type_vecFields::const_iterator iterFind = std::find_if( m_vecFields.begin(), m_vecFields.end(), predicate_FieldHasName<Field>(strFieldNameBeingEdited) );
+    if(iterFind != m_vecFields.end()) //If it was found:
     {
       m_Field_BeingEdited = *iterFind;
 
       //Get new field definition:
       Field fieldNew = get_field_definition(row);
-       
+
       //Change it:
       if(m_Field_BeingEdited != fieldNew) //If it has really changed.
       {
@@ -211,8 +211,9 @@ void Box_DB_Table_Definition::on_adddel_changed(const Gtk::TreeModel::iterator&
        //Refuse to edit field definitions that were not created by glom:
        if(Field::get_glom_type_for_gda_type( m_Field_BeingEdited.get_field_info().get_gdatype() )  == Field::TYPE_INVALID)
        {
-         Gtk::MessageDialog dialog(gettext("This database field was created or edited outside of Glom. It has a data type that is not supported by Glom. Your system administrator may be able to correct this."));
-         dialog.set_transient_for(*get_app_window());
+         Gtk::MessageDialog dialog(gettext("<b>Invalid database structure</b>"), true);
+         dialog.set_secondary_text(gettext("This database field was created or edited outside of Glom. It has a data type that is not supported by Glom. Your system administrator may be able to correct this."));
+         //TODO: dialog.set_transient_for(*get_application());
          dialog.run();
        }
        else
@@ -225,11 +226,11 @@ void Box_DB_Table_Definition::on_adddel_changed(const Gtk::TreeModel::iterator&
 void Box_DB_Table_Definition::on_adddel_edit(const Gtk::TreeModel::iterator& row)
 {
   m_Field_BeingEdited = get_field_definition(row);
-  
+
   m_pDialog->set_field(m_Field_BeingEdited, m_strTableName);
- 
+
   //m_pDialog->set_modified(false); //Disable [Apply] at start.
-  
+
   m_pDialog->show_all();
 }
 
@@ -263,7 +264,7 @@ Field Box_DB_Table_Definition::get_field_definition(const Gtk::TreeModel::iterat
   //DB field definition:
 
   //Start with original definitions, so that we preserve things like UNSIGNED.
-  //TODO maybe use document's fieldinfo instead of m_Fields.
+  //TODO maybe use document's fieldinfo instead of m_vecFields.
   Field field_temp;
   bool test = get_fields_for_table_one_field(m_strTableName, strFieldNameBeforeEdit, field_temp);
   if(test)
@@ -281,7 +282,7 @@ Field Box_DB_Table_Definition::get_field_definition(const Gtk::TreeModel::iterat
     //Type:
     const Glib::ustring& strType = m_AddDel.get_value(row, m_colType);
 
-    Field::glom_field_type glom_type =  Field::get_type_for_name(strType);
+    Field::glom_field_type glom_type =  Field::get_type_for_ui_name(strType);
     Gnome::Gda::ValueType fieldType = Field::get_gda_type_for_glom_type(glom_type);
 
     //Unique:
@@ -297,7 +298,7 @@ Field Box_DB_Table_Definition::get_field_definition(const Gtk::TreeModel::iterat
     //Put it together:
     fieldResult.set_field_info(fieldInfo);
   }
-        
+
   return fieldResult;
 }
 
@@ -311,8 +312,9 @@ void Box_DB_Table_Definition::on_Properties_apply()
     //Refuse to edit field definitions that were not created by glom:
     if( Field::get_glom_type_for_gda_type(m_Field_BeingEdited.get_field_info().get_gdatype()) == Field::TYPE_INVALID )
     {
-      Gtk::MessageDialog dialog(gettext("This database field was created or edited outside of Glom. It has a data type that is not supported by Glom. Your system administrator may be able to correct this."));
-      dialog.set_transient_for(*get_app_window());
+      Gtk::MessageDialog dialog(gettext("<b>Invalid database structure</b>"), true);
+         dialog.set_secondary_text(gettext("This database field was created or edited outside of Glom. It has a data type that is not supported by Glom. Your system administrator may be able to correct this."));
+      //TODO: dialog.set_transient_for(*get_app_window());
       dialog.run();
     }
     else
@@ -331,7 +333,7 @@ void Box_DB_Table_Definition::on_Properties_apply()
 void Box_DB_Table_Definition::change_definition(const Field& fieldOld, Field field)
 {
   Bakery::BusyCursor(*get_app_window());
-    
+
   //DB field defintion:
 
   postgres_change_column(fieldOld, field);
@@ -360,15 +362,15 @@ void Box_DB_Table_Definition::change_definition(const Field& fieldOld, Field fie
    //MySQL does this all with ALTER_TABLE, with "CHANGE" followed by the same details used with "CREATE TABLE",
    //MySQL also makes it easier to change the type.
    // but Postgres uses various subcommands, such as  "ALTER COLUMN", and "RENAME".
-  
-    
+
+ 
   //Extra Glom field definitions:
   Document_Glom* pDoc = static_cast<Document_Glom*>(get_document());
   if(pDoc)
   {
     //Get Table's fields:
     Document_Glom::type_vecFields vecFields = pDoc->get_table_fields(m_strTableName);
-    
+
     //Find old field:
     const Glib::ustring field_name_old = fieldOld.get_name();
     Document_Glom::type_vecFields::iterator iterFind = std::find_if( vecFields.begin(), vecFields.end(), predicate_FieldHasName<Field>(field_name_old) );
@@ -405,7 +407,7 @@ void Box_DB_Table_Definition::change_definition(const Field& fieldOld, Field fie
 
 void Box_DB_Table_Definition::fill_fields()
 {
-  m_Fields = get_fields_for_table(m_strTableName);
+  m_vecFields = get_fields_for_table(m_strTableName);
 }
 
 void  Box_DB_Table_Definition::postgres_add_column(const Field& field, bool not_extras)
@@ -425,19 +427,19 @@ void  Box_DB_Table_Definition::postgres_change_column(const Field& field_old, co
 {
   Gnome::Gda::FieldAttributes field_info = field.get_field_info();
   Gnome::Gda::FieldAttributes field_info_old = field_old.get_field_info();
-  
+
   //If the underlying data type has changed:
   if( field_info.get_gdatype() != field_info_old.get_gdatype() )
   {
     postgres_change_column_type(field_old, field); //This will also change everything else at the same time.
   }
   else
-  {      
+  {
     //Change other stuff, without changing the type:
     postgres_change_column_extras(field_old, field);
   }
 }
-  
+
 void  Box_DB_Table_Definition::postgres_change_column_type(const Field& field_old, const Field& field)
 {
   sharedptr<SharedConnection> sharedconnection = connect_to_server();
@@ -446,7 +448,7 @@ void  Box_DB_Table_Definition::postgres_change_column_type(const Field& field_ol
     Glib::RefPtr<Gnome::Gda::Connection> gda_connection = sharedconnection->get_gda_connection();
 
     bool new_column_created = false;
-    
+
     //If the datatype has changed:
     if(field.get_field_info().get_gdatype() != field_old.get_field_info().get_gdatype())
     {
@@ -464,7 +466,7 @@ void  Box_DB_Table_Definition::postgres_change_column_type(const Field& field_ol
       {
         //TODO: Warn about a delay, and possible loss of precision, before actually doing this.
         //TODO: Try to use a unique name for the temp column:
-     
+
         Field fieldTemp = field;
         fieldTemp.set_name("glom_temp_column");
         postgres_add_column(fieldTemp); //This might also involves several commands.
@@ -481,6 +483,11 @@ void  Box_DB_Table_Definition::postgres_change_column_type(const Field& field_ol
         Glib::ustring conversion_command;
         switch(field.get_glom_type())
         {
+          case Field::TYPE_BOOLEAN: //CAST does not work if the destination type is numeric.
+          {
+            conversion_command = "FALSE"; //TODO: Find a way to convert: "to_number( " + field_old.get_name() + ", '999999999.99' )";
+            break;
+          }
           case Field::TYPE_NUMERIC: //CAST does not work if the destination type is numeric.
           {
             conversion_command = "to_number( " + field_old.get_name() + ", '999999999.99' )";
@@ -502,8 +509,7 @@ void  Box_DB_Table_Definition::postgres_change_column_type(const Field& field_ol
             break;
           }
         }
-          
-          
+
         Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute( "UPDATE " + m_strTableName + " SET  " + fieldTemp.get_name() + " = " + conversion_command );  //TODO: Not full type details.
         if(datamodel)
         {
@@ -599,7 +605,7 @@ void Box_DB_Table_Definition::postgres_change_column_extras(const Field& field_o
 
   }
   */
-    
+ 
    /*
     //If the not-nullness has changed:
     if( set_anyway ||  (field.get_field_info().get_allow_null() != field_old.get_field_info().get_allow_null()) )
@@ -607,9 +613,7 @@ void Box_DB_Table_Definition::postgres_change_column_extras(const Field& field_o
       Glib::ustring nullness = (field.get_field_info().get_allow_null() ? "NULL" : "NOT NULL");
       Query_execute(  "ALTER TABLE " + m_strTableName + " ALTER COLUMN " + field.get_name() + "  SET " + nullness);
     }
-  */
-   
-  
+  */ 
 }
 
 
