@@ -21,7 +21,8 @@
 #include "layoutwidgetbase.h"
 
 LayoutWidgetBase::LayoutWidgetBase()
-: m_pLayoutItem(0)
+: m_pLayoutItem(0),
+  m_pMenuPopup(0)
 {
 }
 
@@ -34,12 +35,13 @@ LayoutWidgetBase::~LayoutWidgetBase()
   }
 }
 
-void LayoutWidgetBase::set_layout_item(LayoutItem* layout_item)
+void LayoutWidgetBase::set_layout_item(LayoutItem* layout_item, const Glib::ustring& table_name)
 {
   if(m_pLayoutItem)
     delete m_pLayoutItem;
 
   m_pLayoutItem = layout_item;
+  m_table_name = table_name;
 }
 
 const LayoutItem* LayoutWidgetBase::get_layout_item() const
@@ -56,3 +58,8 @@ LayoutWidgetBase::type_signal_layout_changed LayoutWidgetBase::signal_layout_cha
 {
   return m_signal_layout_changed;
 }
+
+LayoutWidgetBase::type_signal_layout_item_added LayoutWidgetBase::signal_layout_item_added()
+{
+  return m_signal_layout_item_added;
+}
